from django.contrib import admin

from .models import Document


class DocumentAdmin(admin.ModelAdmin):
    def save_model(self, request, obj, form, change):
        obj.uploader = request.user
        obj.save()


admin.site.register(Document, DocumentAdmin)



import os

from django.conf import settings
from django.core.exceptions import ValidationError
from django.db import models

from bureaucracy import DocxTemplate


def upload_path(inst, fn):
    return os.path.join(settings.MEDIA_ROOT, 'doc_templates', inst.type, fn)


class Document(models.Model):
    template_file = models.FileField(upload_to=upload_path)
    type = models.CharField(max_length=50)
    validate_fields = models.BooleanField(default=True)

    created = models.DateTimeField(auto_now_add=True)
    last_updated = models.DateTimeField(auto_now=True)
    uploader = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, editable=False)

    def render(self, context, format='docx'):
        return DocxTemplate(self.template_file).render(context, format=format)

    def render_to_file(self, context, path, format='docx'):
        DocxTemplate(self.template_file).render_and_save(context, path, format)

    def clean(self):
        if self.validate_fields:
            if not hasattr(settings, 'DOCX_TEMPLATE_VARS'):
                raise ValidationError("Can't validate fields if DOCX_TEMPLATE_VARS is not set.")

            if self.type not in settings.DOCX_TEMPLATE_VARS:
                raise ValidationError(
                    "Specified document type not found."
                    " Either uncheck validate fields or add it to the DOCX_TEMPLATE_VARS setting")

            field_names_in_doc = DocxTemplate(self.template_file).get_field_names()

            if set(settings.DOCX_TEMPLATE_VARS[self.type]) != field_names_in_doc:
                missing = set(settings.DOCX_TEMPLATE_VARS[self.type]) - field_names_in_doc
                superfluous = field_names_in_doc - set(settings.DOCX_TEMPLATE_VARS[self.type])
                raise ValidationError(
                    {'template_file': 'Mailmerge fields in template did not match the ones configured for this type.'
                                      'missing: {missing}, superfluous: {superfluous}'.format(missing=missing,
                                                                                              superfluous=superfluous)})
        else:
            pass



from django.template.backends.django import DjangoTemplates

from bureaucracy.powerpoint.engines import BaseEngine


class DjangoSlideEngine(BaseEngine):

    def __init__(self, extra_builtins=None):
        extra_builtins = extra_builtins or []

        options = {
            'NAME': '',
            'OPTIONS': {
                'builtins': ['django_bureaucracy.templatetags', ] + extra_builtins

            },
            'DIRS': [],
            'APP_DIRS': True
        }

        self.django_tpls = DjangoTemplates(options)

    def render(self, fragment, context):
        fragment = '{{% autoescape off %}}{fragment}{{% endautoescape %}}'.format(fragment=fragment)
        tpl = self.django_tpls.from_string(fragment)
        return tpl.render(context)



import warnings

from bureaucracy.powerpoint.placeholders import (CONTEXT_KEY_FOR_PLACEHOLDER,
                                                 AlreadyRenderedException)
from bureaucracy.powerpoint.slides import (CONTEXT_KEY_FOR_SLIDE,
                                           StopSlideRender)
from django import template
from django.template import TemplateSyntaxError

register = template.Library()


class PPTTagNode(template.Node):
    def render_in_place(self, context, placeholder):
        raise NotImplementedError

    def render(self, context):
        try:
            placeholder = context[CONTEXT_KEY_FOR_PLACEHOLDER]
            self.render_in_place(context, placeholder)
        except KeyError:
            warnings.warn("placeholder was not found in context passed to template tag {}. it will not be "
                          "rendered.".format(self.__class__))
        raise AlreadyRenderedException


class LinkNode(PPTTagNode):
    def __init__(self, url_expr, title_expr):
        self.url_expr = url_expr
        self.title_expr = title_expr

    def render_in_place(self, context, placeholder):
        url, title = self.url_expr.resolve(context), self.title_expr.resolve(context)
        placeholder.insert_link(url, title)


@register.tag
def link(parser, token):
    bits = token.split_contents()
    if len(bits) != 3:
        raise TemplateSyntaxError("The '%r' tag requires two arguments: url and title", bits[0])
    url_expr, title_expr = parser.compile_filter(bits[1]), parser.compile_filter(bits[2])
    return LinkNode(url_expr, title_expr)


class RepeatUntilEmptyNode(PPTTagNode):
    def __init__(self, objects_expr, stop_on_repeat=False):
        self.objects_expr = objects_expr
        self.stop_on_repeat = stop_on_repeat

    def render_in_place(self, context, placeholder):
        objects = self.objects_expr.resolve(context)

        placeholder.remove()

        if objects:
            context[CONTEXT_KEY_FOR_SLIDE].insert_another()
            # repeat happened, flagged to halt rendering here, so send the signal
            if self.stop_on_repeat:
                raise StopSlideRender


@register.tag
def repeatwhile(parser, token):
    bits = token.split_contents()
    if len(bits) < 2:
        raise TemplateSyntaxError("The '%r' tag requires one argument: an iterable to check.")
    if len(bits) == 3 and bits[-1] != 'stop_on_repeat':
        raise TemplateSyntaxError("The second argument to '%r' should be the flag stop_on_repeat", bits[0])
    objects_expr = parser.compile_filter(bits[1])
    return RepeatUntilEmptyNode(objects_expr, len(bits) == 3)


class PopNode(template.Node):
    def __init__(self, objects_expr, var_expr):
        self.objects_expr = objects_expr
        self.var_expr = var_expr

    def render(self, context):
        objects = self.objects_expr.resolve(context)
        varname = self.var_expr.var.var
        if len(objects) > 0:
            context[varname] = objects.pop(0)
        elif varname in context:
            del context[varname]
        return ''


@register.tag
def pop(parser, token):
    bits = token.split_contents()
    if len(bits) != 4:
        raise TemplateSyntaxError(
            "The %(tag)r tag requires an iterable and as-var: "
            "{% %(tag)r my_list as my_item %}", tag=bits[0]
        )

    if bits[2] != 'as':
        raise TemplateSyntaxError("The %r tag must assign with the 'as' token")

    objects_expr = parser.compile_filter(bits[1])
    var_expr = parser.compile_filter(bits[3])
    return PopNode(objects_expr, var_expr)


class TableNode(PPTTagNode):
    @classmethod
    def handle_token(cls, parser, token):
        bits = token.split_contents()

        if len(bits) != 2:
            raise TemplateSyntaxError("Table tag expected exactly one argument, got {}".format(len(bits)))

        return cls(parser.compile_filter(bits[1]))

    def __init__(self, contents_expr):
        self.contents_expr = contents_expr

    def render_in_place(self, context, placeholder):
        table_contents = self.contents_expr.resolve(context)

        n_rows = len(table_contents)

        if not n_rows:
            return ""

        n_cols = len(table_contents[0])
        placeholder = placeholder.insert_table(n_rows, n_cols)

        for row_idx, row_values in enumerate(table_contents):
            for col_idx, cell_value in enumerate(row_values):
                placeholder.table.rows[row_idx].cells[col_idx].text = str(cell_value)

        context[CONTEXT_KEY_FOR_PLACEHOLDER] = placeholder


@register.tag
def table(parser, token):
    return TableNode.handle_token(parser, token)



import os

from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-bureaucracy',
    version='0.3.1',
    packages=['django_bureaucracy'],
    include_package_data=True,
    description='Django layer on top of the bureaucracy package',
    long_description=README,
    install_requires=[
        'django',
        'burocracy>=0.3'
    ],
    author='Maykin Media, Robin Ramael, Sergei Maertens',
    author_email='robin.ramael@maykinmedia.nl,sergei@maykinmedia.nl',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



from bureaucracy import DocxTemplate
from bureaucracy.utils import DOCUMENT_MIME_TYPES
from django.http import HttpResponse

from django_bureaucracy.models import Document


def render_to_download(template, context, format='docx'):
    """
    :param template: a Document, DocxTemplate, path or file-like object representing the template.
    :param context: a dict with the context to render with
    :return:
    """
    if isinstance(template, DocxTemplate):
        doc = template
    elif isinstance(template, Document):
        doc = DocxTemplate(template.template_file.path)
    else:
        doc = DocxTemplate(template)

    data = doc.render(context, format=format)

    return HttpResponse(data, content_type=DOCUMENT_MIME_TYPES[format])
